'use strict';

const objectIdentity = require('object-identity');
const cfw_kv = require('worktop/cfw.kv');
const crypto = require('worktop/crypto');

const make_id = (...key) => key.join('::');
const make = (binding, context) => {
    return (name, handler, options) => {
        const type = options?.type || 'json';
        const lifetimes = {
            ttl: options?.ttl ?? 600,
            maxTtl: options?.maxTtl ?? 1000,
        };
        return new Proxy(handler, {
            async apply(target, this_arg, args_array) {
                const key = args_array.length
                    ? make_id(name, await objectIdentity.identify(args_array, crypto.SHA1))
                    : name;
                const result = await cfw_kv.read(binding, key, {
                    metadata: true,
                    cacheTtl: lifetimes.ttl,
                    // @ts-expect-error TS2769
                    type,
                });
                async function call(date = Date.now()) {
                    const raw_result = await Reflect.apply(target, this_arg, args_array);
                    context.waitUntil(cfw_kv.write(binding, key, raw_result, {
                        metadata: {
                            ttl: date + lifetimes.ttl * 1000,
                        },
                        expirationTtl: lifetimes.maxTtl,
                    }));
                    return raw_result;
                }
                if (result.value != null) {
                    const called_at = Date.now();
                    if (called_at >= result.metadata.ttl)
                        context.waitUntil(call(called_at));
                    return result.value;
                }
                return await call();
            },
        });
    };
};

exports.make = make;
